
#property indicator_separate_window
#property indicator_buffers    2
#property indicator_color1     Silver
#property indicator_color2     Gold
#property indicator_width1     2
#property indicator_width2     1
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern int                FastMa          = 12;
extern ENUM_MA_METHOD     FastMaMode      = MODE_EMA;
extern ENUM_APPLIED_PRICE FastMaPrice     = PRICE_CLOSE;
extern int                SlowMa          = 26;
extern ENUM_MA_METHOD     SlowMaMode      = MODE_EMA;
extern ENUM_APPLIED_PRICE SlowMaPrice     = PRICE_CLOSE;
extern int                SignalMa        = 9;
extern ENUM_MA_METHOD     SignalMaMode    = MODE_EMA;
extern bool               alertsOn        = true;
extern bool               alertsOnCurrent = false;
extern bool               alertsMessage   = true;
extern bool               alertsSound     = true;
extern bool               alertsNotify    = false;
extern bool               alertsEmail     = false;
extern string             soundFile       = "alert2.wav"; 
extern bool               ShowArrows      = false;
extern string             arrowsIdentifier= "macd Arrows";
extern double             arrowsUpperGap  = 0.5;
extern double             arrowsLowerGap  = 0.5;
extern color              arrowsUpColor   = LimeGreen;
extern color              arrowsDnColor   = Red;
extern color              arrowsUpCode    = 241;
extern color              arrowsDnCode    = 242;

//
//
//
//
//

double macd[];
double sign[];
double slope[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(3);
   SetIndexBuffer(0,macd); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,sign);
   SetIndexBuffer(2,slope);
      
   FastMa   = MathMax(FastMa,1);
   SlowMa   = MathMax(SlowMa,1);
   SignalMa = MathMax(SignalMa,1);
   IndicatorShortName("macd("+FastMa+","+SlowMa+","+SignalMa+")");
return(0);
}
int deinit()
{
  deleteArrows(); 
 return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //
  
   for (i=limit; i>=0; i--) macd[i] = iMA(NULL,0,FastMa,0,FastMaMode,FastMaPrice,i) - iMA(NULL,0,SlowMa,0,SlowMaMode,SlowMaPrice,i);
   for (i=limit; i>=0; i--) 
   {    
       sign[i] = iMAOnArray(macd,Bars,SignalMa,0,SignalMaMode,i);  
       slope[i] = slope[i+1];
          if (macd[i]>macd[i+1]) slope[i]  =  1;
          if (macd[i]<macd[i+1]) slope[i]  = -1;
          
          //
          //
          //
          //
          //
     
          if (ShowArrows)
          {
             deleteArrow(Time[i]);
             if (slope[i]!= slope[i+1])
             {
                if (slope[i] == 1)  drawArrow(i,arrowsUpColor,arrowsUpCode,false);
                if (slope[i] ==-1)  drawArrow(i,arrowsDnColor,arrowsDnCode, true);
             }
          }
   
 } 
 
 //
 //
 //
 //
 //
 
 if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (slope[whichBar] != slope[whichBar+1])
      {
         if (slope[whichBar] == 1)   doAlert(whichBar,"sloping up");
         if (slope[whichBar] ==-1)   doAlert(whichBar,"sloping down");
      }         
   }
return(0);
} 

//
//
//
//
//

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

           message =  StringConcatenate(Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Macd "+doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " Macd "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
    string name = arrowsIdentifier+":"+Time[i];
    double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}

